package org.apidesign.apifest08.currency;

/** This is the skeleton class for your API. You need to make it public, so
 * it is accessible to your client code (currently in Task1Test.java) file.
 * <p>
 * Feel free to create additional classes or rename this one, just keep all
 * the API and its implementation in this package. Do not spread it outside
 * to other packages.
 */
public final class Convertor {
    private String currency1;
    private String currency2;
    private Rate rate;
    
    Convertor(String currency1, String currency2, Rate rate) {
        if ((currency1 == null)||(currency2 == null)||(rate == null)) {
            throw new IllegalArgumentException("All arguments have to be non-null.");
        }
        this.currency1 = currency1;
        this.currency2 = currency2;
        this.rate = rate;
    }

    public double convert(String fromCurrency, String toCurrency, int amount) {
        if ((fromCurrency == null)||(toCurrency == null)) {
            throw new IllegalArgumentException("All arguments have to be non-null.");            
        }
        
        if (currency1.equals(fromCurrency) && currency2.equals(toCurrency)) {
            return rate.convertAtoB(amount);
        } else if (currency2.equals(fromCurrency) && currency1.equals(toCurrency)) {
            return rate.convertBtoA(amount);
        } else {
            throw new IllegalArgumentException("Convertor " + this.toString() + 
                    " cannot work with currencies " + fromCurrency + " and " + toCurrency + ".");
        }
    }

    public double convert(String fromCurrency, String toCurrency, double amount) {
        if ((fromCurrency == null)||(toCurrency == null)) {
            throw new IllegalArgumentException("All arguments have to be non-null.");            
        }
        
        if (currency1.equals(fromCurrency) && currency2.equals(toCurrency)) {
            return rate.convertAtoB(amount);
        } else if (currency2.equals(fromCurrency) && currency1.equals(toCurrency)) {
            return rate.convertBtoA(amount);
        } else {
            throw new IllegalArgumentException("Convertor " + this.toString() + 
                    " cannot work with currencies " + fromCurrency + " and " + toCurrency + ".");
        }
    }
        
    @Override
    public String toString() {
        return currency1+currency2;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Convertor other = (Convertor) obj;
        if (this.currency1 != other.currency1 && (this.currency1 == null || !this.currency1.equals(other.currency1))) {
            return false;
        }
        if (this.currency2 != other.currency2 && (this.currency2 == null || !this.currency2.equals(other.currency2))) {
            return false;
        }
        if (this.rate != other.rate && (this.rate == null || !this.rate.equals(other.rate))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 67 * hash + (this.currency1 != null ? this.currency1.hashCode() : 0);
        hash = 67 * hash + (this.currency2 != null ? this.currency2.hashCode() : 0);
        hash = 67 * hash + (this.rate != null ? this.rate.hashCode() : 0);
        return hash;
    }
}
